@@ -10,11 +10,11 @@
  * Author       : Andy Loening                                             *
  *                                                                         *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-/* $Id: m-conc.h,v 1.23 2007/05/21 20:16:11 enlf Exp $
+/* $Id: m-conc.h,v 1.27 2008/03/06 11:38:34 enlf Exp $
  */
 
 /*
-   Copyright (C) 1997-2007 by Erik Nolf
+   Copyright (C) 1997-2008 by Erik Nolf
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -43,6 +43,16 @@
                               D E F I N E S
 ****************************************************************************/
 
+/* Acquisition scanner modality (integer) */
+typedef enum {
+  MDC_CONC_MODALITY_UNKNOWN = -1,
+  MDC_CONC_MODALITY_PET = 0,
+  MDC_CONC_MODALITY_CT = 1,
+  MDC_CONC_MODALITY_SPECT = 2,
+  MDC_CONC_MODALITY_LAST = 3, /* place holder, as list begins at -1 */
+  MDC_CONC_NUM_MODALITIES = 4
+} MdcConcModalityTypes;
+
 /* concorde data file types (integer) */
 typedef enum {
   MDC_CONC_FILE_UNKNOWN,
@@ -54,6 +64,13 @@ typedef enum {
   MDC_CONC_FILE_BLANK,
   MDC_CONC_FILE_RESERVED, /* 7 is skipped for some reason */
   MDC_CONC_FILE_MU_MAP,
+  MDC_CONC_FILE_SCATTER_CORRECTION,
+  MDC_CONC_FILE_CRYSTAL_EFFICIENCY_DATA,
+  MDC_CONC_FILE_CRYSTAL_INTERFERENCE_CORRECTION,
+  MDC_CONC_FILE_TRANSAXIAL_GEOMETRIC_CORRECTION,
+  MDC_CONC_FILE_AXIAL_GEOMETRIC_CORRECTION,
+  MDC_CONC_FILE_CT_PROJECTION_DATA,
+  MDC_CONC_FILE_SPECT_RAW_PROJECTION_DATA,
   MDC_CONC_NUM_FILE_TYPES
 } MdcConcFileTypes;
 
@@ -68,12 +85,18 @@ typedef enum {
   MDC_CONC_ACQ_SINGLES,                    /* Singles transmission acquisition */
   MDC_CONC_ACQ_WINDOWED_COINCIDENCE,       /* Windowed coincidence transmission acquisition */
   MDC_CONC_ACQ_NON_WINDOWED_COINCIDENCE,   /* Non-windowed coincidence transmission acquisition */
+  MDC_CONC_ACQ_CT_PROJECTION,              /* CT projection acquisition */
+  MDC_CONC_ACQ_CT_CALIBRATION,             /* CT calibration acquisition */
+  MDC_CONC_ACQ_SPECT_PLANAR_PROJECTION,    /* SPECT planar projection acquisition */
+  MDC_CONC_ACQ_SPECT_MULTIPROJECTION,      /* SPECT multi-projection acquisition */
+  MDC_CONC_ACQ_SPECT_CALIBRATION,          /* SPECT calibration acquisition */
   MDC_CONC_NUM_ACQ_MODES
 } MdcConcAcqModes;
 
 typedef enum {
   MDC_CONC_BED_MOTION_STATIC,              /* Static or unknown bed motion */
   MDC_CONC_BED_MOTION_CONTINOUS,           /* Continous bed motion */
+  MDC_CONC_BED_MOTION_MULTIPLE,            /* Multiple bed positions, i.e. step and shoot */
   MDC_CONC_NUM_BED_MOTIONS
 } MdcConcBedMotion;
 
@@ -236,9 +259,51 @@ typedef enum {
   /* Version of header parameters (float) */
   MDC_CONC_HDR_VERSION,
 
-  /* Scanner model (integer) */
+  /* Manufacturer's name (string) */
+  MDC_CONC_HDR_MANUFACTURER,
+
+  /* Scanner model (integer) 
+     0 - Unknown 
+     2000 - Primate
+     2001 - Rodent 
+     2002 - microPET2
+     2500 - Focus_220
+     2501 - Focus_120 
+     3000 - mCAT 
+     3500 - mCATII
+     4000 - mSPECT
+     5000 - Inveon_Dedicated_PET
+     5001 - Inveon_MM_Platform
+     6000 - MR_PET_Head_Insert
+     8000 - Tuebingen_PET_MR */
   MDC_CONC_HDR_MODEL,
 
+  /* Acquisition scanner modality (integer)
+     -1 - Unknown acquisition modality
+      0 - PET acquisition
+      1 - CT acquisition
+      2 - SPECT acquisition */
+  MDC_CONC_HDR_MODALITY,
+
+  /* Scanner modality configuration number (integer)
+     0 - Unknown
+     2000 - Primate
+     2001 - Rodent
+     2002 - microPET2
+     2500 - Focus_220
+     2501 - Focus_120
+     3000 - mCAT
+     3500 - mCATII
+     3600 - Inveon_MM_Std_CT
+     3601 - Inveon_MM_HiRes_Std_CT
+     3602 - Inveon_MM_Std_LFOV_CT
+     3603 - Inveon_MM_HiRes_LFOV_CT
+     5000 - Inveon_Dedicated_PET
+     6000 - MR_PET_Head_Insert
+     8000 - Tuebingen_PET_MR
+     5500 - Inveon_MM_PET */
+  MDC_CONC_HDR_MODALITY_CONFIGURATION,
+
   /* Institution identification (string) */
   MDC_CONC_HDR_INSTITUTION,
 
@@ -250,34 +315,64 @@ typedef enum {
   /*       line, up to the EOL, is used after the parameter name. */
   MDC_CONC_HDR_FILE_NAME,
 
-  /* Data file type (integer) */
-  /*   0 - Unknown data file type */
-  /*   1 - List mode data file */
-  /*   2 - Sinogram data file */
-  /*   3 - Normalization data file */
-  /*   4 - Attenuation correction data file */
-  /*   5 - Image data file */
-  /*   6 - Blank data file */
-  /*   8 - Mu map data file */
+  /* Data file type (integer)
+   0 - Unknown data file type
+   1 - List mode data file
+   2 - Sinogram data file
+   3 - Normalization data file
+   4 - Attenuation correction data file
+   5 - Image data file
+   6 - Blank data file
+   8 - Mu map data file
+   9 - Scatter correction data file
+   10 - Crystal efficiency data
+   11 - Crystal interference correction
+   12 - Transaxial geometric correction
+   13 - Axial geometric correction
+   14 - CT projection data
+   15 - SPECT raw projection data */
   MDC_CONC_HDR_FILE_TYPE,
 
-  /* Acquisition mode (integer) */
-  /* 0 - Unknown acquisition mode */
-  /*   1 - Blank acquisition */
-  /*   2 - Emission acquisition */
-  /*   3 - Dynamic acquisition */
-  /*   4 - Gated acquisition */
-  /*   5 - Continuous bed motion acquisition */
-  /*   6 - Singles transmission acquisition */
-  /*   7 - Windowed coincidence transmission acquisition */
-  /*   8 - Non-windowed coincidence transmission acquisition */
+  /* Acquisition mode (integer)
+   0 - Unknown acquisition mode
+   1 - Blank acquisition
+   2 - Emission acquisition
+   3 - Dynamic acquisition
+   4 - Gated acquisition
+   5 - Continuous bed motion acquisition
+   6 - Singles transmission acquisition
+   7 - Windowed coincidence transmission acquisition
+   8 - Non-windowed coincidence transmission acquisition
+   9 - CT projection acquisition
+   10 - CT calibration acquisition
+   11 - SPECT planar projection acquisition
+   12 - SPECT multi-projection acquisition
+   13 - SPECT calibration acquisition */
   MDC_CONC_HDR_ACQUISITION_MODE,               
 
+  /* Bed control (integer)
+     0 - Unknown bed control
+     1 - Dedicated PET
+     2 - microCAT II
+     3 - Multimodality bed control
+     4 - microPET bed control */
+  MDC_CONC_HDR_BED_CONTROL,
+
   /* Bed motion (integer) */
   /*   0 - Static or unknown bed motion */
   /*   1 - Continuous bed motion */
+  /*   2 - Multiple bed positions, i.e. step and shoot */
   MDC_CONC_HDR_BED_MOTION,
 
+  /* Number of bed positions in data file (integer) */
+  MDC_CONC_HDR_NUMBER_BED_POSITIONS,
+
+  /* Horizontal bed calibration, in microns (float) */
+  MDC_CONC_HDR_HORIZONTAL_BED_CALIBRATION,
+
+  /* Vertical bed calibration, in microns (float) */
+  MDC_CONC_HDR_VERTICAL_BED_CALIBRATION,
+
   /* Number of frames in data file (integer) */
   MDC_CONC_HDR_TOTAL_FRAMES,
 
@@ -354,6 +449,9 @@ typedef enum {
   /* Axial plane size, in cm (float) */
   MDC_CONC_HDR_AXIAL_PLANE_SIZE,
 
+  /* Number of detector panels ("rings/CT" are "1") (integer) */
+  MDC_CONC_HDR_NUMBER_DETECTOR_PANELS,
+
   /* Lower level energy threshold, in KeV (float) */
   MDC_CONC_HDR_LLD,
 
@@ -585,11 +683,43 @@ typedef enum {
   /* Zoom applied to data set (float) */
   MDC_CONC_HDR_ZOOM,
 
+  /* X origin of volume, in voxels (integer) */
+  MDC_CONC_HDR_VOLUME_ORIGIN_X,
+
+  /* Y origin of volume, in voxels (integer) */
+  MDC_CONC_HDR_VOLUME_ORIGIN_Y,
+
+  /* Z origin of volume, in voxels (integer) */
+  MDC_CONC_HDR_VOLUME_ORIGIN_Z,
+
+  /* Registration data available (integer)
+     0 - No registration data available
+     1 - CT registration data available
+     2 - PET registration data available */
+  MDC_CONC_HDR_REGISTRATION_AVAILABLE,
+
+  /* Transformation matrix filename, possibly including path (string)
+     NOTE: Filename may contain spaces, therefore the ENTIRE 
+     line, up to the EOL, is used after the parameter name. */
+  MDC_CONC_HDR_TRANSFORMATION_MATRIX,
+
+  /* Spatial identification for registration (string) */
+  MDC_CONC_HDR_SPATIAL_IDENTIFIER,
+
   /* Reconstructed pixel size, in cm (float) */
   /*     NOTE: pixel_size = (((X_crystal_pitch / 2.0) * X_dim) / */
   /*           (image_size * zoom)) * (effective_radius / radius) */
   MDC_CONC_HDR_PIXEL_SIZE,
 
+  /* Reconstructed pixel size in X, in mm (float) */
+  MDC_CONC_HDR_PIXEL_SIZE_X,
+
+  /* Reconstructed pixel size in Y, in mm (float) */
+  MDC_CONC_HDR_PIXEL_SIZE_Y,
+
+  /* Reconstructed pixel size in Z, in mm (float) */
+  MDC_CONC_HDR_PIXEL_SIZE_Z,
+
   /* Calibration units (integer) */
   /*   0 - Unknown calibration units */
   /*   1 - nanoCuries/cc */
@@ -618,10 +748,29 @@ typedef enum {
   /* Study identification (string) */
   MDC_CONC_HDR_STUDY_IDENTIFIER,
 
+  /* Acquisition user ID (string) */
+  MDC_CONC_HDR_ACQUISITION_USER_ID,
+
+  /* Histogram user ID (string) */
+  MDC_CONC_HDR_HISTOGRAM_USER_ID,
+ 
+  /* Reconstruction user ID (string) */
+  MDC_CONC_HDR_RECONSTRUCTION_USER_ID,
+
+  /* Scatter correction user ID (string) */
+  MDC_CONC_HDR_SCATTER_CORRECTION_USER_ID,
+ 
+  /* Acquisition notes (string) */
+  MDC_CONC_HDR_ACQUISITION_NOTES,
+
   /* Scan start date and time (string) */
   /* Format is: Sun Sep 16 01:03:52 1973 */
   MDC_CONC_HDR_SCAN_TIME,
 
+  /* Scan start date and time - GMT-based (string)
+     Format is: Sun Sep 16 01:03:52 1973 */
+  MDC_CONC_HDR_GMT_SCAN_TIME,
+
   /* Injected compound (string) */
   MDC_CONC_HDR_INJECTED_COMPOUND,
 
@@ -643,6 +792,26 @@ typedef enum {
   /* DO NOT include injection decay correction factor. */
   MDC_CONC_HDR_INJECTION_DECAY_CORRECTION,
 
+  /* Pre- and residual activity units (integer)
+     0 - Unknown dose units
+     1 - mCi
+     2 - MBq */
+  MDC_CONC_HDR_ACTIVITY_UNITS,
+
+  /* Activity before injection (float) */
+  MDC_CONC_HDR_ACTIVITY_BEFORE_INJECTION,
+
+  /* Activity before injection measurement date and time (string)
+     Format is: Sun Sep 16 01:03:52 1973 */
+  MDC_CONC_HDR_ACTIVITY_BEFORE_INJECTION_TIME,
+  
+  /* Residual activity (float) */
+  MDC_CONC_HDR_RESIDUAL_ACTIVITY,
+
+  /* Residual activity measurement date and time (string)
+     Format is: Sun Sep 16 01:03:52 1973 */
+  MDC_CONC_HDR_RESIDUAL_ACTIVITY_TIME,
+
   /* Number of gating inputs in study (integer) */
   /* NOTE: This is ONLY present when gating inputs are present. */
   MDC_CONC_HDR_GATE_INPUTS,
@@ -719,6 +888,216 @@ typedef enum {
   /* Water access (string) */
   MDC_CONC_HDR_WATER_ACCESS,
 
+  /* Subject date of birth (string) */
+  MDC_CONC_HDR_SUBJECT_DOB,
+  
+  /* Subject age (string) */
+  MDC_CONC_HDR_SUBJECT_AGE,
+  
+  /*Subject sex (string) */
+  MDC_CONC_HDR_SUBJECT_SEX,
+
+  /* Subject scan region (string) */
+  MDC_CONC_HDR_SUBJECT_SCAN_REGION,
+
+  /* Subject glucose level (string) */
+  MDC_CONC_HDR_SUBJECT_GLUCOSE_LEVEL,
+
+  /* Subject glucose level measurement time (string) */
+  MDC_CONC_HDR_SUBJECT_GLUCOSE_LEVEL_TIME,
+
+  /* Original acquisition filename, DOES NOT INCLUDE path (string)
+    NOTE: Filename may contain spaces, therefore the ENTIRE 
+    line, up to the EOL, is used after the parameter name. */
+  MDC_CONC_HDR_ACQUISITION_FILE_NAME,
+
+
+  /* Following parameters are used for CT or SPECT modalities */
+
+
+  /* Gantry rotation (integer) 
+     0 (FALSE) - Gantry does NOT rotate
+     !0 (TRUE) - Gantry does rotate */
+  MDC_CONC_HDR_GANTRY_ROTATION,
+
+  /* Rotation direction (integer) 
+     0 - Clockwise
+     1 - Counterclockwise */
+  MDC_CONC_HDR_ROTATION_DIRECTION,
+
+  /* Rotating gantry starting angle, in degrees (float) */
+  MDC_CONC_HDR_ROTATING_STAGE_START_POSITION,
+
+  /* Rotating gantry stop angle, in degrees (float) */
+  MDC_CONC_HDR_ROTATING_STAGE_STOP_POSITION,
+
+  /* Number of rotation projections for rotating gantry (integer) */
+  MDC_CONC_HDR_NUMBER_OF_PROJECTIONS,
+
+
+  /* Following parameters are used for CT modality only */
+
+  /* CAT file version (integer) */
+  MDC_CONC_HDR_CT_FILE_VERSION,
+
+  /* Header size of CAT files before dark and light projections (integer) */
+  MDC_CONC_HDR_CT_HEADER_SIZE,
+
+  /* CT transaxial projection size, in pixels (integer) */
+  MDC_CONC_HDR_CT_PROJ_SIZE_TRANSAXIAL,
+
+  /* CT axial projection size, in pixels (integer) */
+  MDC_CONC_HDR_CT_PROJ_SIZE_AXIAL,
+
+  /* Number to average the dark calibration projections (integer) */
+  MDC_CONC_HDR_CT_AVERAGE_DARK_PROJECTIONS,
+
+  /* Number to average the light calibration projections (integer) */
+  MDC_CONC_HDR_CT_AVERAGE_LIGHT_PROJECTIONS,
+
+  /* Total positions to acquire the light calibration projections (integer) */
+  MDC_CONC_HDR_CT_LIGHT_CALIBRATION_PROJECTIONS,
+
+  /* Indicates if the positions to acquire light projections are same as scan projection positions (integer) */
+  MDC_CONC_HDR_CT_DEPENDENT_LIGHT_CALIBRATION_PROJECTIONS,
+
+  /* CT X-ray detector offset, in mm (float) */
+  MDC_CONC_HDR_CT_XRAY_DETECTOR_OFFSET,
+
+  /* CT detector transaxial position, in cm (float) */
+  MDC_CONC_HDR_CT_DETECTOR_TRANSAXIAL_POSITION,
+
+  /* CT detector uncropped transaxial pixels (integer) */
+  MDC_CONC_HDR_CT_UNCROPPED_TRANSAXIAL_PIXELS,
+
+  /* CT detector uncropped axial pixels (integer) */
+  MDC_CONC_HDR_CT_UNCROPPED_AXIAL_PIXELS,
+
+  /* CT detector cropped transaxial pixels (integer) */
+  MDC_CONC_HDR_CT_CROPPED_TRANSAXIAL_PIXELS,
+
+  /* CT detector cropped axial pixels (integer) */
+  MDC_CONC_HDR_CT_CROPPED_AXIAL_PIXELS,
+
+  /* CT X-ray detector pitch, in um (float) */
+  MDC_CONC_HDR_CT_XRAY_DETECTOR_PITCH,
+
+  /* CT horizontal rotation-axis-bed angle, in degrees (float) */
+  MDC_CONC_HDR_CT_HORIZ_ROT_AXIS_BED_ANGLE,
+
+  /* CT vertical rotation-axis-bed angle, in degrees (float) */
+  MDC_CONC_HDR_CT_VERT_ROT_AXIS_BED_ANGLE,
+
+  /* CT exposure time, in msecs (float) */
+  MDC_CONC_HDR_CT_EXPOSURE_TIME,
+
+  /* CT total scan time, in min:sec (integer:integer) */
+  MDC_CONC_HDR_CT_SCAN_TIME,
+
+  /* CT warping type (integer)
+   0 - Unknown warping type
+   1 - No warping
+   2 - Bilinear warping
+   3 - Nearest neighbor warping */
+  MDC_CONC_HDR_CT_WARPING,
+
+  /* CT defect map filename, possibly including path (string)
+    NOTE: Filename may contain spaces, therefore the ENTIRE 
+    line, up to the EOL, is used after the parameter name. */
+  MDC_CONC_HDR_CT_DEFECT_MAP_FILE_NAME,
+
+  /* CT x-ray voltage, in kVp (float) */
+  MDC_CONC_HDR_CT_XRAY_VOLTAGE,
+
+  /* CT anode current, in uA (float) */
+  MDC_CONC_HDR_CT_ANODE_CURRENT,
+
+  /* Number of CT calibration exposures (integer) */
+  MDC_CONC_HDR_CT_CALIBRATION_EXPOSURES,
+
+  /* CT cone angle, in degrees (float) */
+  MDC_CONC_HDR_CT_CONE_ANGLE,
+
+  /* CT projection interpolation type (integer)
+   0 - Unknown projection interpolation type
+   1 - Bilinear projection interpolation
+   2 - Nearest neighbor projection interpolation */
+  MDC_CONC_HDR_CT_PROJECTION_INTERPOLATION,
+
+  /* CT source to detector distance, in cm (float) */
+  MDC_CONC_HDR_CT_SOURCE_TO_DETECTOR,
+
+  /* CT source to center of rotation, in cm (float) */
+  MDC_CONC_HDR_CT_SOURCE_TO_CROT,
+
+  /* CT vetical detector offset, in pixels (float) */
+  MDC_CONC_HDR_CT_DETECTOR_VERTICAL_OFFSET,
+
+  /* CT detector tilt relative to horizontal axis, in degrees (float) */
+  MDC_CONC_HDR_CT_DETECTOR_HORIZONTAL_TILT,
+
+  /* CT detector tilt relative to vertical axis, in degrees (float) */
+  MDC_CONC_HDR_CT_DETECTOR_VERTICAL_TILT,
+
+  /* CT axial projection bin factor, in pixels (integer) */
+  MDC_CONC_HDR_CT_TRANSAXIAL_BIN_FACTOR,
+
+  /* CT axial projection bin factor, in pixels (integer) */
+  MDC_CONC_HDR_CT_AXIAL_BIN_FACTOR,
+
+  /* CT gating signal used (integer) */
+  MDC_CONC_HDR_CT_GATING,
+
+  /* CT Hounsfield scale (float) */
+  MDC_CONC_HDR_CT_HOUNSFIELD_SCALE,
+
+  /* CT Hounsfield offset (float) */
+  MDC_CONC_HDR_CT_HOUNSFIELD_OFFSET,
+
+  /* CT projection downsample factor (integer) */
+  MDC_CONC_HDR_CT_PROJ_DOWNSAMPLE_FACTOR,
+
+  /* CT first projection used in reconstruction (integer) */
+  MDC_CONC_HDR_CT_FIRST_RECON_PROJ,
+
+  /* CT last projection used in reconstruction (integer) */
+  MDC_CONC_HDR_CT_LAST_RECON_PROJ,
+
+  /* CT every Nth projection used for reconstruction (integer) */
+  MDC_CONC_HDR_CT_RECON_EVERY_NTH_PROJ,
+
+  /* CT attenuation of water, in cm^-1 (float) */
+  MDC_CONC_HDR_CT_ATTENUATION_WATER,
+
+  /* CT TX rotation offsets: X Y Z, in mm (float) */
+  MDC_CONC_HDR_CT_TX_ROTATION_OFFSETS,
+
+  /* CT TX transaxial offsets: X Y Z, in mm (float) */
+  MDC_CONC_HDR_CT_TX_TRANSAXIAL_OFFSETS,
+
+  /* CT BH correction applied (int)
+     0 (FALSE) - CT BH correction has NOT been applied
+     !0 (TRUE) - CT BH correction has been applied */
+  MDC_CONC_HDR_CT_BH_CORRECTION,
+
+  /* CT aluminum filter thickness (mm) (float) */
+  MDC_CONC_HDR_CT_ALUMINUM_FILTER_THICKNESS,
+  
+  /* CT projection array (integer  float)
+     projection_number    acquisition_position (degrees) */
+  MDC_CONC_HDR_PROJECTION,
+
+  /* CT projection average center offset (float) */
+  MDC_CONC_HDR_CT_PROJECTION_AVERAGE_CENTER_OFFSET,
+
+  /* CT projection average center offset array (integer  float)
+     projection_number    average center offset (mm) */
+  MDC_CONC_HDR_CT_PROJECTION_CENTER_OFFSET,
+
+  /* CT projection horizontal bed offset array (integer  float)
+     projection_number    horizontal bed offset (mm) */
+  MDC_CONC_HDR_CT_PROJECTION_HORIZONTAL_BED_OFFSET,
+
   /* End of Header indicator */
   MDC_CONC_HDR_END_OF_HEADER,
 
@@ -752,6 +1131,9 @@ typedef enum {
   /* Ending horizontal bed offset, in cm (float) */
   MDC_CONC_BLOCK_ENDING_BED_OFFSET,
 
+  /* Number of bed passes during frame (integer) */
+  MDC_CONC_BLOCK_BED_PASSES,
+
   /* Vertical bed offset, in cm (float) */
   MDC_CONC_BLOCK_VERTICAL_BED_OFFSET,
 
@@ -820,7 +1202,8 @@ const char *MdcLoadPlaneCONC(FILEINFO *fi, int img);
 const char *MdcLoadHeaderCONC(FILEINFO *fi);
 const char *MdcLoadCONC(FILEINFO *fi);
 const char *MdcSavePlaneCONC(FILEINFO *fi, int img);
-const char *MdcSaveHeaderCONC(FILEINFO *fi);
+const char *MdcSaveInitCONC(FILEINFO *fi, char *raw_filename);
+const char *MdcSaveHeaderCONC(FILEINFO *fi, char *raw_filename);
 const char *MdcSaveCONC(FILEINFO *fi);
 
 int MdcCheckCONC(FILEINFO *fi);
