@@ -18,11 +18,11 @@
  *                - Mosaic support     - Roland Marcus Rutschmann          *
  *                                                                         *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-/* $Id: m-dicm.c,v 1.180 2007/10/22 19:47:53 enlf Exp $
+/* $Id: m-dicm.c,v 1.182 2008/01/23 21:28:35 enlf Exp $
  */
 
 /*
-   Copyright (C) 1997-2007 by Erik Nolf
+   Copyright (C) 1997-2008 by Erik Nolf
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -226,7 +226,7 @@ char *MdcDicomHandleImages(FILEINFO *fi, MDC_DICOM_STUFF_T *dicom, IMAGE *image,
           if ((img == 0) && (f == 0)) 
             fi->pat_slice_orient = MdcTryPatSliceOrient(fi->pat_orient);
           if (fi->pat_slice_orient != MDC_UNKNOWN) {
-            MdcFillImgPos(fi,img,img%fi->dim[3],0.0);
+            MdcFillImgPos(fi,img,fi->dim[3]==0 ? 0 : img%fi->dim[3],0.0);
             MdcFillImgOrient(fi,img); 
           }
         }
