@@ -47,11 +47,11 @@
  *                MdcGetHeartRate()     - Get heart rate from gated data   *
  *                                                                         *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-/* $Id: m-files.c,v 1.107 2007/11/09 23:03:50 enlf Exp $ 
+/* $Id: m-files.c,v 1.109 2008/02/27 21:07:40 enlf Exp $ 
  */
 
 /*
-   Copyright (C) 1997-2007 by Erik Nolf
+   Copyright (C) 1997-2008 by Erik Nolf
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -1012,16 +1012,18 @@ void MdcNewExt(char dest[], char *src, char *ext)
 /* create a prefix of the form: 000 ...  ... 999,A00................ZZZ
                                   <- normal ->  |  <----- extra ----->
                                      (1000)              (33696)
+
 normal prefixes =
-      integer value from 000 to 999      (1000)
-extra  prefixes =                          +
-      first char: A...Z                   (26)
-     second char: 0...9,A...Z             (36)
-     third  char: 0...9,A...Z             (36)
-With this construction we keep the numeric prefixes, 
-added with an alphabetical number to create  even more
-names without overlapping. An 'ls' gives the filenames
-in sequence as they were created. */
+     integers from 000 to 999        (1000)
+
+extra  prefixes =                        +
+     1st char: A...Z                   (26)
+     2nd char: 0...9,A...Z             (36)
+     3rd char: 0...9,A...Z             (36)
+
+This construct gives at first numeric prefixes, extended with
+a lot more alphanumeric prefixes before overlap. A directory
+listing will thus show the filenames in sequence of creation. */
         
 void MdcPrefix(int n)
 {
