@@ -8,8 +8,13 @@
  * project      : (X)MedCon by Erik Nolf                                   *
  *                                                                         *
  * Functions    : MdcCheckGIF()              - Check for GIF format        *
- *                MdcDoExtensions()          - Handle extensions           *
  *                MdcReadGIF()               - Read GIF file               *
+ *                MdcDoExtension()           - Handle extensions           *
+ *                MdcReadGifHeader()         - Read header block           *
+ *                MdcReadGifImageBlk()       - Read image block            *
+ *                MdcReadGifControlBlk()     - Read control block          *
+ *                MdcReadGifPlainTextBlk()   - Read plain text block       *
+ *                MdcReadGifApplicationBlk() - Read application block      *
  *                MdcUnpackImage()           - Unpack LZW compressed image *
  *                MdcPutGifLine()            - Copy line to memory buffer  *
  *                MdcWriteGIF()              - Write GIF file              *
@@ -36,11 +41,11 @@
  *                We only write GIF89a animated gifs                       *
  *                                                                         *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-/* $Id: m-gif.c,v 1.1.1.1 2000/10/28 16:51:20 enlf Exp $
+/* $Id: m-gif.c,v 1.37 2008/01/23 21:28:36 enlf Exp $
  */
 
 /*
-   Copyright (C) 1997-2000 by Erik Nolf
+   Copyright (C) 1997-2008 by Erik Nolf
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -60,30 +65,43 @@
                               H E A D E R S
 ****************************************************************************/
 
+#include "m-depend.h"
+
+#include <stdio.h>
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+#ifndef _WIN32
+#include <strings.h>
+#endif
+#endif
+
 #include "medcon.h"
 
 /****************************************************************************
                               D E F I N E S 
 ****************************************************************************/
 
-#define SIG_GIF         "GIF"
-
 #define largest_code    4095            /* largest possible code */
 #define table_size      5003            /* table dimensions */
 
-Uint8 code_buffer[259];         /* where the codes go */
-Int16 oldcode[table_size];      /* the table */
-Int16 currentcode[table_size];
-Uint8 newcode[table_size];
+static Uint8 code_buffer[259];         /* where the codes go */
+static Int16 oldcode[table_size];      /* the table */
+static Int16 currentcode[table_size];
+static Uint8 newcode[table_size];
 
-Int16 code_size;
-Int16 clear_code;
-Int16 eof_code;
-Int16 bit_offset;
-Int16 byte_offset;
-Int16 bits_left;
-Int16 max_code;
-Int16 free_code;
+static Int16 code_size;
+static Int16 clear_code;
+static Int16 eof_code;
+static Int16 bit_offset;
+static Int16 byte_offset;
+static Int16 bits_left;
+static Int16 max_code;
+static Int16 free_code;
 
 
 /****************************************************************************
@@ -99,14 +117,176 @@ int MdcCheckGIF(FILEINFO *fi)
 
   if ( fread((char *)&gh,1,MDC_GIF_GH_SIZE,fi->ifp) != MDC_GIF_GH_SIZE ) 
     return(MDC_BAD_READ);
-  if ( memcmp(gh.sig,SIG_GIF,3) ) 
-    return(MDC_FRMT_BAD);
+  if ( memcmp(gh.sig,MDC_GIF_SIG,3) ) 
+    return(MDC_FRMT_NONE);
 
   return(MDC_FRMT_GIF);
 } 
   
+/* unpack a GIF file */
+char *MdcReadGIF(FILEINFO *fi)
+{
+  FILE *fp=fi->ifp;
+  MDC_GIFHEADER gh;
+  MDC_GIFIMAGEBLOCK iblk;
+  Uint32 b, c, img=0, number=0;
+  char *err=NULL;
+  Uint32 bytes;
+
+  if (MDC_PROGRESS) MdcProgress(MDC_PROGRESS_BEGIN,0.,"Reading GIF:");
+
+  if (MDC_VERBOSE) MdcPrntMesg("GIF  Reading <%s> ...",fi->ifname);
+
+  if (MDC_ECHO_ALIAS == MDC_YES) {
+    MdcEchoAliasName(fi); return(NULL); /* a little pointless for GIF */
+  }
+
+  /* initialize structures */
+  memset(&gh,0,MDC_GIF_GH_SIZE);
+  memset(&iblk,0,MDC_GIF_IBLK_SIZE);
+
+  /* put some defaults we use */
+  fi->endian=MDC_FILE_ENDIAN=MDC_LITTLE_ENDIAN; /* always for a GIF */
+  fi->dim[0] = 4; fi->dim[4]=1;
+
+  /* make sure it's a GIF file */
+  if ( MdcReadGifHeader(fp,&gh) != MDC_YES ) return("GIF  Bad read gifheader");
+  if ( memcmp(gh.sig, MDC_GIF_SIG, 3) ) return("No GIF file");
+
+  if (MDC_INFO) {
+    MdcPrntScrn("GIFHEADER (%d bytes)\n",MDC_GIF_GH_SIZE);
+    MdcPrintLine('-',MDC_HALF_LENGTH);
+  }
+
+  fi->bits=8; /* (gh.flags & 0x0007) + 1; */
+
+  if (MDC_INFO) { 
+    MdcPrntScrn("signature: %.6s\n",gh.sig);
+    MdcPrntScrn("screen width: %d\nscreen height: %d\n",
+                          gh.screenwidth,gh.screenheight);
+    MdcPrntScrn("global palette: ");
+  }
+
+  /* get colour map if there is one */
+  if (gh.flags & 0x80) {
+    if (fi->map < 2) fi->map=MDC_MAP_PRESENT;
+    c = 3 * (1 << ((gh.flags & 0x0007) + 1));
+    if (fread(fi->palette,1,c,fp) != c) 
+      return("GIF  Bad read global palette");
+ 
+    if (MDC_INFO) {
+      MdcPrntScrn("Yes\n");
+      MdcPrntScrn("bits: %hd\n",(gh.flags & 7)+1 );
+      MdcPrntScrn("colors: %hd\n",(1 << ((gh.flags & 0x0007)+1)));
+      MdcPrntScrn("sorted: ");
+      if (gh.flags > 0x0008 ) MdcPrntScrn("Yes\n");
+      else MdcPrntScrn("No\n");
+      MdcPrntScrn("background: %hd\n", (int)gh.background);
+      MdcPrntScrn("aspect: %hd\n\n",(int)gh.aspect);
+    }
+   
+  } else if (MDC_INFO) MdcPrntScrn("No\n");
+
+  /* step through the blocks */
+  while((c=(Int16)fgetc(fp))==',' || c=='!' || c==0) {
+
+    /* if it's an image block... */
+    if (c == ',') {
+
+      if (MDC_PROGRESS) MdcProgress(MDC_PROGRESS_SET,0.0,NULL);
+ 
+      if (MDC_INFO) {
+        MdcPrntScrn("\n");
+        MdcPrintLine('=',MDC_HALF_LENGTH);
+        MdcPrntScrn("\nIMAGEBLOCK %03d (%d bytes)\n",number+1
+                                                    ,MDC_GIF_IBLK_SIZE);
+        MdcPrintLine('-',MDC_HALF_LENGTH);
+      }
+
+      /* get the start of the image block */
+      if (MdcReadGifImageBlk(fp,&iblk) != MDC_YES)
+        return("GIF  Bad read imageblock");
+
+      number+=1;
+
+      if (!MdcGetStructID(fi,number)) return("GIF  Bad malloc IMG_DATA struct");
+       
+      /* fill in the IMG_DATA struct */
+      fi->image[img].width=(Uint32) iblk.width;
+      fi->image[img].height=(Uint32) iblk.height;
+
+      if (MDC_INFO) {
+        MdcPrntScrn("image left: %hu\n",iblk.left);
+        MdcPrntScrn("image top: %hu\n",iblk.top);
+        MdcPrntScrn("image width: %hu\n",iblk.width);
+        MdcPrntScrn("image height: %hu\n",iblk.height);
+        MdcPrntScrn("interlaced: ");
+        if ( iblk.flags & 0x0040 ) MdcPrntScrn("Yes\n");
+        else MdcPrntScrn("No\n");
+        MdcPrntScrn("local palette: ");
+      }
+      /* get the local colour map if there is one */
+      if (iblk.flags & 0x80) {
+        if (fi->map < 2) fi->map=MDC_MAP_PRESENT;
+        if (MDC_INFO) {
+          MdcPrntScrn("Yes\n");
+          MdcPrntScrn("sorted: ");
+          if ( iblk.flags & 0x0020 ) MdcPrntScrn("Yes\n");
+          else MdcPrntScrn("No\n");
+        }           
+        b = 3*(1<<((iblk.flags & 0x0007) + 1));
+        if (fread(fi->palette,1,b,fp) != b) 
+          return("GIF  Bad read local palette");
+      } else if (MDC_INFO) MdcPrntScrn("No\n");
+
+      /* get the initial code size */
+      if ((c=(Int16)fgetc(fp))==EOF) return("GIF  Bad read initial code");
+
+      fi->image[img].bits = c;
+      fi->image[img].flags = iblk.flags;
+
+      /* get an image buffer */
+      
+      bytes=MdcPixels2Bytes(fi->image[img].width*fi->image[img].height*8);
+          
+      if ( (fi->image[img].buf=MdcGetImgBuffer(bytes)) == NULL )
+        return("GIF  Bad malloc image buffer");
+
+      if (MDC_PROGRESS) MdcProgress(MDC_PROGRESS_SET,0.5,NULL);
+
+      /* unpack the image */
+      err = MdcUnpackImage(fi,img);
+
+      if (MDC_PROGRESS) MdcProgress(MDC_PROGRESS_SET,1.0,NULL);
+
+      /* fill in the FILEINFO struct */
+      fi->bits=fi->image[img].bits=8;
+      fi->image[img].type=BIT8_U;
+      fi->dim[3] = number;
+      img = number;
+
+      /* warning if there was an error */
+      if (err != NULL) {
+        MdcPrntWarn(err);
+        err = MdcHandleTruncated(fi,fi->number,MDC_NO);
+        if (err != NULL) return(err);
+        break;
+      }
+
+    }else if (c == '!') MdcDoExtension(fi);  
+
+  }
+
+  if (fi->number == 0) return("GIF  No valid images found");
+
+  MdcCloseFile(fi->ifp);
+
+  if (fi->truncated) return("GIF  Truncated image file");
+
+  return NULL;
+}
 /* this function is called when the GIF decoder encounters an extension */
-void DoExtension(FILEINFO *fi)
+void MdcDoExtension(FILEINFO *fi)
 {
   FILE *fp=fi->ifp;
   MDC_GIFPLAINTEXT pt;
@@ -118,24 +298,24 @@ void DoExtension(FILEINFO *fi)
   memset(&cb,0,MDC_GIF_CBLK_SIZE);
   memset(&ap,0,MDC_GIF_ABLK_SIZE);
 
-  if (MDC_INFO) { printf("\n"); MdcPrintLine('=',MDC_HALF_LENGTH); }
+  if (MDC_INFO) { MdcPrntScrn("\n"); MdcPrintLine('=',MDC_HALF_LENGTH); }
  
   switch (c=(Int16)fgetc(fp)) {
 
    case 0x0001:		/* plain text descriptor */
-    if (fread((char *)&pt,1,MDC_GIF_TBLK_SIZE,fp)  == MDC_GIF_TBLK_SIZE ) {
-
-      MdcSWAP(pt.left); MdcSWAP(pt.top);
-      MdcSWAP(pt.gridwidth); MdcSWAP(pt.gridheight);
+    if (MdcReadGifPlainTextBlk(fp,&pt) == MDC_YES) {
 
       if (MDC_INFO) { 
-        printf("\nPLAIN TEXT BLOCK\n"); MdcPrintLine('-', MDC_HALF_LENGTH);
-        printf("This block requires %u bytes\n",pt.blocksize);
-        printf("Text location at (%u,%u)\n",pt.left,pt.top);
-        printf("Grid dimensions are %u by %u\n",pt.gridwidth,pt.gridheight);
-        printf("Cell dimensions are %u by %u\n",pt.cellwidth,pt.cellheight);
-        printf("Foregound colour is %u\n",pt.forecolour);
-        printf("Background colour is %u\n",pt.backcolour);
+        MdcPrntScrn("\nPLAIN TEXT BLOCK\n"); 
+          MdcPrintLine('-', MDC_HALF_LENGTH);
+        MdcPrntScrn("This block requires %u bytes\n",pt.blocksize);
+        MdcPrntScrn("Text location at (%u,%u)\n",pt.left,pt.top);
+        MdcPrntScrn("Grid dimensions are %u by %u\n",pt.gridwidth
+                                                       ,pt.gridheight);
+        MdcPrntScrn("Cell dimensions are %u by %u\n",pt.cellwidth
+                                                       ,pt.cellheight);
+        MdcPrntScrn("Foregound colour is %u\n",pt.forecolour);
+        MdcPrntScrn("Background colour is %u\n",pt.backcolour);
       }
       do {
 	if ((n=(Int16)fgetc(fp)) != EOF) {
@@ -145,58 +325,46 @@ void DoExtension(FILEINFO *fi)
           }
 	}
       }while (n > 0 && n != EOF);
-    } else {
-#if XSUPPORTED
-      if (XMDC_MEDCON) {
-        XMdcDisplayErr("GIF  Bad read plain text block");
-      }else
-#endif
+    }else{
       MdcPrntWarn("GIF  Bad read plain text block");
     }
     break;
 
    case 0x00f9:		/* graphic control block */
-    if (fread((char *)&cb,1,MDC_GIF_CBLK_SIZE,fp) == MDC_GIF_CBLK_SIZE) {
-
-      MdcSWAP(cb.delay);
+    if (MdcReadGifControlBlk(fp,&cb) == MDC_YES) {
 
       if (MDC_INFO) {
-        printf("\nCONTROL BLOCK\n"); MdcPrintLine('-',MDC_HALF_LENGTH);
-        printf("This block requires %u bytes\n",cb.blocksize);
+        MdcPrntScrn("\nCONTROL BLOCK\n"); MdcPrintLine('-',MDC_HALF_LENGTH);
+        MdcPrntScrn("This block requires %u bytes\n",cb.blocksize);
         switch((cb.flags >> 2) & 0x0007) {
 	  case 0:
-	    printf("No disposal specified\n");
+	    MdcPrntScrn("No disposal specified\n");
 	    break;
 	  case 1:
-	    printf("Do not dispose\n");
+	    MdcPrntScrn("Do not dispose\n");
 	    break;
 	  case 2:
-	    printf("Dispose to background colour\n");
+	    MdcPrntScrn("Dispose to background colour\n");
 	    break;
 	  case 3:
-	    printf("Dispose to previous graphic\n");
+	    MdcPrntScrn("Dispose to previous graphic\n");
 	    break;
 	  default:
-	    printf("Unknown disposal procedure\n");
+	    MdcPrntScrn("Unknown disposal procedure\n");
 	    break;
         }
         if (cb.flags & 0x0002)
-	  printf("User input required - delay for %g seconds\n"
+	  MdcPrntScrn("User input required - delay for %g seconds\n"
                                       ,(float)cb.delay/100.);
         else
-	  printf("No user input required\n");
+	  MdcPrntScrn("No user input required\n");
 
         if (cb.flags & 0x0001)
-	  printf("Transparent colour: %u\n",cb.transparent_colour);
+	  MdcPrntScrn("Transparent colour: %u\n",cb.transparent_colour);
         else
-	  printf("No transparent_colour\n");
+	  MdcPrntScrn("No transparent_colour\n");
       }
-    } else {
-#if XSUPPORTED
-      if (XMDC_MEDCON) {
-        XMdcDisplayErr("GIF  Bad read control block");
-      }else
-#endif
+    }else{ 
       MdcPrntWarn("GIF  Bad read control block");
     }
     break;
@@ -204,21 +372,14 @@ void DoExtension(FILEINFO *fi)
    case 0x00fe:		/* comment extension */
       
     if (MDC_INFO) { 
-      printf("\nCOMMENT BLOCK\n"); MdcPrintLine('-',MDC_HALF_LENGTH); 
+      MdcPrntScrn("\nCOMMENT BLOCK\n"); MdcPrintLine('-',MDC_HALF_LENGTH); 
     }
     do {
       if ((n=(Int16)fgetc(fp)) != EOF) {
         if (n > 0) {
           fi->comment = (char *)MdcRealloc(fi->comment,fi->comm_length+n+2);
           if (fi->comment == NULL) {
-#if XSUPPORTED
-            if (XMDC_MEDCON) {
-              XMdcDisplayErr("Couldn't allocate comment buffer");
-            }else
-#endif
-            {
             MdcPrntWarn("Couldn't allocate comment buffer");
-            }
           }else if (fi->comm_length==0) fi->comment[fi->comm_length] = '\0';
         } 
 	for (i=0;i<n;++i) {
@@ -237,16 +398,16 @@ void DoExtension(FILEINFO *fi)
     break;
 
    case 0x00ff:		/* application extension */
-    if (fread((char *)&ap,1,MDC_GIF_ABLK_SIZE,fp) == MDC_GIF_ABLK_SIZE) {
+    if (MdcReadGifApplicationBlk(fp,&ap) == MDC_YES) {
       if (MDC_INFO) {
-        printf("\nAPPLICATION BLOCK\n"); MdcPrintLine('-',MDC_HALF_LENGTH);
-        printf("This block requires %d bytes\n",ap.blocksize);
-        printf("Identification string: %.8s\n",ap.applstring);
-        printf("Authentication string: %.3s\n",ap.authentication);
+        MdcPrntScrn("\nAPPLICATION BLOCK\n"); MdcPrintLine('-',MDC_HALF_LENGTH);
+        MdcPrntScrn("This block requires %d bytes\n",ap.blocksize);
+        MdcPrntScrn("Identification string: %.8s\n",ap.applstring);
+        MdcPrntScrn("Authentication string: %.3s\n",ap.authentication);
       }
       do {
         if ((n=(Int16)fgetc(fp)) != EOF) {
-          if (MDC_INFO) printf("\nSub-block requires %d bytes:\n",n);
+          if (MDC_INFO) MdcPrntScrn("\nSub-block requires %d bytes:\n",n);
 	  for (i=0;i<n;++i) {
              c = (Int16)fgetc(fp); 
              if (MDC_INFO) MdcPrintChar(c);
@@ -254,21 +415,11 @@ void DoExtension(FILEINFO *fi)
 	}
       }while (n > 0 && n != EOF);
     }else{
-#if XSUPPORTED
-      if (XMDC_MEDCON) {
-        XMdcDisplayErr("GIF  Bad read application block");
-      }else
-#endif
       MdcPrntWarn("GIF  Bad read application block");
     }
     break;
 
    default:		/* something else */
-#if XSUPPORTED
-    if (XMDC_MEDCON) {
-      XMdcDisplayWarn("GIF  Unknown extension 0x%02.2x\n",c & 0x00ff);
-    }else
-#endif 
     MdcPrntWarn("GIF  Unknown extension 0x%02.2x\n",c & 0x00ff);
     n=(Int16)fgetc(fp);
     for (i=0;i<n;++i) fgetc(fp);
@@ -277,193 +428,83 @@ void DoExtension(FILEINFO *fi)
 
 }
 
-/* unpack a GIF file */
-char *MdcReadGIF(FILEINFO *fi)
+int MdcReadGifHeader(FILE *fp, MDC_GIFHEADER *gh)
 {
-  FILE *fp=fi->ifp;
-  MDC_GIFHEADER gh;
-  MDC_GIFIMAGEBLOCK iblk;
-  Uint32 b, c, img=0, images=0;
-  char *err=NULL;
-  Uint32 bytes;
+  char bb[MDC_GIF_GH_SIZE];
 
-#if XSUPPORTED
-  if (XMDC_MEDCON) XMdcBeginProgressBar("Reading GIF:");
-#endif
+  if (fread(bb,1,MDC_GIF_GH_SIZE,fp) != MDC_GIF_GH_SIZE ) return(MDC_NO);
 
-  if (MDC_VERBOSE) MdcPrntMesg("GIF  Reading <%s> ...",fi->ifname);
-
-  memset(&gh,0,MDC_GIF_GH_SIZE);
-  memset(&iblk,0,MDC_GIF_IBLK_SIZE);
-
-  fi->endian=MDC_FILE_ENDIAN=MDC_LITTLE_ENDIAN; /* always for a GIF */
-  fi->dim[0] = 4; fi->dim[4]=1;
-
-  /* make sure it's a GIF file */
-  if (fread((char *)&gh,1,MDC_GIF_GH_SIZE,fp) != MDC_GIF_GH_SIZE) 
-    return("GIF  Bad read gifheader");
-  if ( memcmp(gh.sig, SIG_GIF, 3) ) return("No GIF file");
-
-  MdcSWAP(gh.screenwidth); MdcSWAP(gh.screenheight);
-
-  if (MDC_INFO) {
-    printf("GIFHEADER (%d bytes)\n",MDC_GIF_GH_SIZE);
-    MdcPrintLine('-',MDC_HALF_LENGTH);
-  }
-
-  fi->bits=8; /* (gh.flags & 0x0007) + 1; */
-
-  if (MDC_INFO) { 
-    printf("signature: %.6s\n",gh.sig);
-    printf("screen width: %d\nscreen height: %d\n",
-                          gh.screenwidth,gh.screenheight);
-    printf("global palette: ");
-  }
-
-  /* get colour map if there is one */
-  if (gh.flags & 0x80) {
-    if (fi->map < 2 )  fi->map = MDC_MAP_PRESENT;
-    c = 3 * (1 << ((gh.flags & 0x0007) + 1));
-    if (fread(fi->palette,1,c,fp) != c) 
-      return("GIF  Bad read global palette");
- 
-    if (MDC_INFO) {
-      printf("Yes\n");
-      printf("bits: %hd\n",(gh.flags & 7)+1 );
-      printf("colors: %hd\n",(1 << ((gh.flags & 0x0007)+1)));
-      printf("sorted: ");
-      if (gh.flags > 0x0008 ) printf("Yes\n");
-      else printf("No\n");
-      printf("background: %hd\n", (int)gh.background);
-      printf("aspect: %hd\n\n",(int)gh.aspect);
-    }
-   
-  } else if (MDC_INFO) printf("No\n");
-
-  /* step through the blocks */
-  while((c=(Int16)fgetc(fp))==',' || c=='!' || c==0) {
-
-    /* if it's an image block... */
-    if (c == ',') {
-
-#if XSUPPORTED
-      if (XMDC_MEDCON) {
-        pvalue = 0.0;
-        XMdcUpdateProgressBar(NULL);
-      }
-#endif
- 
-      if (MDC_INFO) {
-        printf("\n");
-        MdcPrintLine('=',MDC_HALF_LENGTH);
-        printf("\nIMAGEBLOCK %03d (%d bytes)\n",images+1,MDC_GIF_IBLK_SIZE);
-        MdcPrintLine('-',MDC_HALF_LENGTH);
-      }
-
-      /* get the start of the image block */
-      if (fread(&iblk,1,MDC_GIF_IBLK_SIZE,fp) != MDC_GIF_IBLK_SIZE)
-	return("GIF  Bad read imageblock");
-
-
-      MdcSWAP(iblk.left);  MdcSWAP(iblk.top);
-      MdcSWAP(iblk.width); MdcSWAP(iblk.height);
-
-      fi->number = ++images;
-
-      if (!MdcGetStructID(fi)) return("GIF  Bad malloc IMG_DATA struct");
-       
-      /* fill in the IMG_DATA struct */
-      fi->image[img].width=(Uint32) iblk.width;
-      fi->image[img].height=(Uint32) iblk.height;
-
-      if (MDC_INFO) {
-        printf("image left: %hu\n",iblk.left);
-        printf("image top: %hu\n",iblk.top);
-        printf("image width: %hu\n",iblk.width);
-        printf("image height: %hu\n",iblk.height);
-        printf("interlaced: ");
-        if ( iblk.flags & 0x0040 ) printf("Yes\n");
-        else printf("No\n");
-        printf("local palette: ");
-      }
-      /* get the local colour map if there is one */
-      if (iblk.flags & 0x80) {
-        if (fi->map < 2) fi->map = MDC_MAP_PRESENT;
-        if (MDC_INFO) {
-          printf("Yes\n");
-          printf("sorted: ");
-          if ( iblk.flags & 0x0020 ) printf("Yes\n");
-          else printf("No\n");
-        }           
-        b = 3*(1<<((iblk.flags & 0x0007) + 1));
-        if (fread(fi->palette,1,b,fp) != b) 
-          return("GIF  Bad read local palette");
-      } else if (MDC_INFO) printf("No\n");
-
-      /* get the initial code size */
-      if ((c=(Int16)fgetc(fp))==EOF) return("GIF  Bad read initial code");
-
-      fi->image[img].bits = c;
-      fi->image[img].flags = iblk.flags;
-
-      /* get an image buffer */
-      
-      bytes=MdcPixels2Bytes(fi->image[img].width*fi->image[img].height*8);
-          
-      if ( (fi->image[img].buf=MdcGetImgBuffer(bytes)) == NULL )
-        return("GIF  Bad malloc image buffer");
-
-#if XSUPPORTED
-      if (XMDC_MEDCON) {
-        pvalue = 0.5;
-        XMdcUpdateProgressBar(NULL);
-      }
-#endif
-
-      /* unpack the image */
-      err = MdcUnpackImage(fi,img);
-
-#if XSUPPORTED
-      if (XMDC_MEDCON) {
-        pvalue = 1.0;
-        XMdcUpdateProgressBar(NULL);
-      }
-#endif
-
-      /* fill in the FILEINFO struct */
-      fi->bits=fi->image[img].bits=8;
-      fi->image[img].type=BIT8_U;
-      fi->dim[3] = images;
-      img = images;
-
-      /* warning if there was an error */
-      if (err != NULL) {
-#if XSUPPORTED
-        if (XMDC_MEDCON) {
-          XMdcDisplayErr(err);
-        }else
-#endif
-        MdcPrntWarn(err);
-        err = MdcHandleTruncated(fi,fi->number,MDC_NO);
-        if (err != NULL) return(err);
-        break;
-      }
-
-    }else if (c == '!') DoExtension( fi);  
-
-  }
-
-  if (fi->number == 0) return("GIF  No valid images found");
-
-  if ((fi->map < 2) && MDC_MAKE_GRAY) fi->map=MDC_MAP_GRAY;
-
-  MdcFileClose(fi->ifp);
-
-  if (fi->truncated) return("GIF  Truncated image file");
-
-  return NULL;
+  memcpy(gh->sig          ,bb  ,6);
+  memcpy(&gh->screenwidth ,bb+6,2); MdcSWAP(gh->screenwidth);
+  memcpy(&gh->screenheight,bb+8,2); MdcSWAP(gh->screenheight);
+  gh->flags      = (Uint8) bb[10];
+  gh->background = (Uint8) bb[11];
+  gh->aspect     = (Uint8) bb[12];
+  
+  return(MDC_YES);
 }
 
+int MdcReadGifImageBlk(FILE *fp, MDC_GIFIMAGEBLOCK *ib)
+{
+  char bb[MDC_GIF_IBLK_SIZE];
+
+  if (fread(bb,1,MDC_GIF_IBLK_SIZE,fp) != MDC_GIF_IBLK_SIZE) return(MDC_NO);
+
+  memcpy(&ib->left  ,bb  ,2); MdcSWAP(ib->left);
+  memcpy(&ib->top   ,bb+2,2); MdcSWAP(ib->top);
+  memcpy(&ib->width ,bb+4,2); MdcSWAP(ib->width);
+  memcpy(&ib->height,bb+6,2); MdcSWAP(ib->height);
+  ib->flags = (Uint8) bb[8];
+ 
+  return(MDC_YES);
+}
+
+int MdcReadGifControlBlk(FILE *fp, MDC_GIFCONTROLBLOCK *cb)
+{
+  char bb[MDC_GIF_CBLK_SIZE];
+
+  if (fread(bb,1,MDC_GIF_CBLK_SIZE,fp) != MDC_GIF_CBLK_SIZE) return(MDC_NO);
+
+  cb->blocksize = (Uint8) bb[0];
+  cb->flags     = (Uint8) bb[1];
+  memcpy(&cb->delay,bb+2,2); MdcSWAP(cb->delay);
+  cb->transparent_colour = (Uint8) bb[4];
+  cb->terminator = (Uint8) bb[5];
+
+  return(MDC_YES);
+}
+
+int MdcReadGifPlainTextBlk(FILE *fp, MDC_GIFPLAINTEXT *pt)
+{
+  char bb[MDC_GIF_TBLK_SIZE];
+
+  if (fread(bb,1,MDC_GIF_TBLK_SIZE,fp) != MDC_GIF_TBLK_SIZE) return(MDC_NO);
+
+  pt->blocksize = (Uint8) bb[0];
+  memcpy(&pt->left      ,bb+1,2); MdcSWAP(pt->left);
+  memcpy(&pt->top       ,bb+3,2); MdcSWAP(pt->top);
+  memcpy(&pt->gridwidth ,bb+5,2); MdcSWAP(pt->gridwidth);
+  memcpy(&pt->gridheight,bb+7,2); MdcSWAP(pt->gridheight);
+  pt->cellwidth  = (Uint8) bb[9];
+  pt->cellheight = (Uint8) bb[10];
+  pt->forecolour = (Uint8) bb[11];
+  pt->backcolour = (Uint8) bb[12];
+  
+  return(MDC_YES);
+}
+
+int MdcReadGifApplicationBlk(FILE *fp, MDC_GIFAPPLICATION *ap)
+{
+  char bb[MDC_GIF_ABLK_SIZE];
+
+  if (fread(bb,1,MDC_GIF_ABLK_SIZE,fp) != MDC_GIF_ABLK_SIZE) return(MDC_NO);
+
+  ap->blocksize = (Uint8) bb[0];
+  memcpy(ap->applstring    ,bb+1,8);
+  memcpy(ap->authentication,bb+9,3);
+  
+  return(MDC_YES);
+}
 
 /* unpack an LZW compressed image */
 char *MdcUnpackImage(FILEINFO *fi, Uint32 nr )
@@ -471,8 +512,7 @@ char *MdcUnpackImage(FILEINFO *fi, Uint32 nr )
   FILE *fp=fi->ifp;
   Int16 bits=fi->image[nr].bits;
   IMG_DATA *id=&fi->image[nr];
-  Uint8 *pal=fi->palette;
-  Uint8 pix, rd, gr, bl;
+  Uint8 pix;
 
   Int16 bits2;         /* Bits plus 1 */
   Int16 codesize;      /* Current code size in bits */
@@ -523,7 +563,7 @@ char *MdcUnpackImage(FILEINFO *fi, Uint32 nr )
   for (;;) {
     if (bitsleft==8) {
       if (++p >= q && (((blocksize = (Int16)fgetc(fp)) < 1) ||
-	   (q=(p=b)+fread(b,1,blocksize,fp))< (b+blocksize))) {
+	   (q=(p=b)+fread(b,1,(unsigned)blocksize,fp))< (b+blocksize))) {
 	MdcFree(linebuffer);
 	return("GIF  Unexpected EOF (1)");
       }
@@ -536,7 +576,7 @@ char *MdcUnpackImage(FILEINFO *fi, Uint32 nr )
 
     }else {
       if (++p >= q && (((blocksize = (Int16)fgetc(fp)) < 1) ||
-	   (q=(p=b)+fread(b,1,blocksize,fp)) < (b+blocksize))) {
+	   (q=(p=b)+fread(b,1,(unsigned)blocksize,fp)) < (b+blocksize))) {
 	MdcFree(linebuffer);
 	return("GIF  Unexpected EOF (2)");
       }
@@ -544,7 +584,7 @@ char *MdcUnpackImage(FILEINFO *fi, Uint32 nr )
       if (currentcode <= 16) *p >>= (bitsleft=currentcode-8);
       else {
 	  if (++p >= q && (((blocksize = (Int16)fgetc(fp)) < 1) ||
-	       (q=(p=b) + fread(b,1,blocksize,fp)) < (b+blocksize))) {
+	       (q=(p=b) + fread(b,1,(unsigned)blocksize,fp)) < (b+blocksize))) {
 	    MdcFree(linebuffer);
 	    return("GIF  Unexpected EOF (3)");
 	  }
@@ -587,12 +627,8 @@ char *MdcUnpackImage(FILEINFO *fi, Uint32 nr )
     oldtoken = thiscode;
 
     do {
-      if (MDC_MAKE_GRAY) {
-        rd=pal[thiscode*3]; gr=pal[thiscode*3+1]; bl=pal[thiscode*3+2];
-        pix = (Uint8)MdcGRAY(rd,gr,bl);
-      }else{
-        pix = (Uint8)thiscode;
-      }
+
+      pix = (Uint8)thiscode;
 
       linebuffer[nxtbyte++]=pix;
       if (nxtbyte >= id->width) {
@@ -640,9 +676,12 @@ char *MdcWriteGIF(FILEINFO *fi)
 
   MDC_FILE_ENDIAN = MDC_LITTLE_ENDIAN; /* always for a gif */
 
+  /* check supported */
+  if (fi->type == COLRGB) return("GIF  True color files unsupported");
+
   memset(&opt,0,sizeof(MDC_GIFOPT));
 
-  if (MDC_GIF_OPTIONS == MDC_YES && XMDC_MEDCON == MDC_NO) {
+  if ((MDC_GIF_OPTIONS == MDC_YES) && (XMDC_GUI == MDC_NO)) {
     MdcGetGifOpt(fi,&opt);
   }else {
     opt.transp = MDC_NO;
@@ -650,40 +689,32 @@ char *MdcWriteGIF(FILEINFO *fi)
     opt.delay  = GIF_DELAY;
   }
 
-  if (XMDC_MEDCON == MDC_NO)
-    MdcDefaultName(MDC_FRMT_GIF,fi->ofname,fi->ifname);
+  if (XMDC_GUI == MDC_NO) {
+    MdcDefaultName(fi,MDC_FRMT_GIF,fi->ofname,fi->ifname);
+  }
+
+  if (MDC_PROGRESS) MdcProgress(MDC_PROGRESS_BEGIN,0.,"Writing GIF:");
 
-#if XSUPPORTED
-  if (XMDC_MEDCON) XMdcBeginProgressBar("Writing GIF:");
-#endif
   if (MDC_VERBOSE) MdcPrntMesg("GIF  Writing <%s> ...",fi->ofname);
 
-  if (MdcFileExists(fi->ofname))
-    return("GIF  File exists!!");
-  if ( (fi->ofp=fopen(fi->ofname,"wb")) == NULL ) 
-    return ("GIF  Couldn't open file");
+  if (MDC_FILE_STDOUT == MDC_YES) {
+    fi->ofp = stdout;
+  }else{
+    if (MdcKeepFile(fi->ofname))
+      return("GIF  File exists!!");
+    if ( (fi->ofp=fopen(fi->ofname,"wb")) == NULL ) 
+      return ("GIF  Couldn't open file");
+  }
 
   if (MDC_FORCE_INT != MDC_NO) {
     if (MDC_FORCE_INT != BIT8_U) {
-#if XSUPPORTED
-      if (XMDC_MEDCON) {
-        XMdcDisplayWarn("GIF  Only Uint8 pixels supported");
-      }else
-#endif
       MdcPrntWarn("GIF  Only Uint8 pixels supported");
     }
   } 
 
   /* check supported things */
   if (MDC_QUANTIFY || MDC_CALIBRATE) {
-#if XSUPPORTED
-    if (XMDC_MEDCON) {
-      XMdcDisplayWarn("GIF  Normalization loses quantified values!");
-    }else
-#endif
-    {
     MdcPrntWarn("GIF  Normalization loses quantified values!");
-    }
   }
 
   if ( MdcWriteGifHeader(fi,&opt) ) 
@@ -705,12 +736,8 @@ char *MdcWriteGIF(FILEINFO *fi)
   }
 
   for ( nr=0; nr<fi->number; nr++) {
-#if XSUPPORTED
-     if (XMDC_MEDCON) {
-       pvalue += 1./(float)fi->number;
-       XMdcUpdateProgressBar(NULL);
-     }
-#endif
+
+     if (MDC_PROGRESS) MdcProgress(MDC_PROGRESS_INCR,1./(float)fi->number,NULL);
 
      if ( (fi->number > 1) || (opt.transp == MDC_YES)) 
        if ( MdcWriteControlBlock(fi,&opt,nr) ) 
@@ -737,7 +764,7 @@ char *MdcWriteGIF(FILEINFO *fi)
 
   if ( fputc(';',fi->ofp) == EOF ) return("GIF  Bad write terminator code");
 
-  MdcFileClose(fi->ofp);
+  MdcCloseFile(fi->ofp);
 
   return NULL;
 
@@ -748,46 +775,48 @@ void MdcGetGifOpt(FILEINFO *fi, MDC_GIFOPT *opt)
  opt->loop   = MDC_YES;
  opt->transp = MDC_YES;
 
+ if (MDC_FILE_STDIN == MDC_YES) return; /* stdin already in use */
+
  MdcPrintLine('-',MDC_FULL_LENGTH);
- printf("\tGIF OPTIONS\t\tORIG FILE: %s\n",fi->ifname);
+ MdcPrntScrn("\tGIF OPTIONS\t\tORIG FILE: %s\n",fi->ifname);
  MdcPrintLine('-',MDC_FULL_LENGTH);
- printf("\n\tSelect color map:\n\n");
- printf("\t\t%d  ->  present\n",MDC_MAP_PRESENT);
- printf("\t\t%d  ->  gray\n",MDC_MAP_GRAY);
- printf("\t\t%d  ->  rainbow\n",MDC_MAP_RAINBOW);
- printf("\t\t%d  ->  combined\n",MDC_MAP_COMBINED);
- printf("\t\t%d  ->  hotmetal\n",MDC_MAP_HOTMETAL);
- printf("\t\t%d  ->  loaded LUT\n",MDC_MAP_LOADED);
- printf("\n\tYour choice [%hu]? ",fi->map);
+ MdcPrntScrn("\n\tSelect color map:\n\n");
+ MdcPrntScrn("\t\t%d  ->  present\n",MDC_MAP_PRESENT);
+ MdcPrntScrn("\t\t%d  ->  gray\n",MDC_MAP_GRAY);
+ MdcPrntScrn("\t\t%d  ->  rainbow\n",MDC_MAP_RAINBOW);
+ MdcPrntScrn("\t\t%d  ->  combined\n",MDC_MAP_COMBINED);
+ MdcPrntScrn("\t\t%d  ->  hotmetal\n",MDC_MAP_HOTMETAL);
+ MdcPrntScrn("\t\t%d  ->  loaded LUT\n",MDC_MAP_LOADED);
+ MdcPrntScrn("\n\tYour choice [%d]? ",(int)fi->map);
 
  if (!MdcPutDefault(mdcbufr)) {
-   fi->map = (Uint16)atoi(mdcbufr);
-   MdcGetColorMap(fi->map,fi->palette); 
+   fi->map = (Uint8)atoi(mdcbufr);
+   MdcGetColorMap((int)fi->map,fi->palette); 
  }
 
  if (fi->number > 1) {
-   printf("\n\tInsert a display loop    [yes]? ");
+   MdcPrntScrn("\n\tInsert a display loop    [yes]? ");
    mdcbufr[0]='y';
    if (!MdcPutDefault(mdcbufr))
      if (mdcbufr[0]=='n' || mdcbufr[0]=='N') opt->loop = MDC_NO; 
-   printf("\n\tDelay 1/100ths of a second [%3d]? ", GIF_DELAY);
+   MdcPrntScrn("\n\tDelay 1/100ths of a second [%3d]? ", GIF_DELAY);
    if (!MdcPutDefault(mdcbufr)) {
      opt->delay = (Uint16)atoi(mdcbufr);
    }else{
      opt->delay = GIF_DELAY;
    }
  }
- printf("\n\tInsert transparent color [yes]? ");
+ MdcPrntScrn("\n\tInsert transparent color [yes]? ");
  mdcbufr[0]='y';
  if (!MdcPutDefault(mdcbufr))
    if (mdcbufr[0]=='n' || mdcbufr[0]=='N') opt->transp = MDC_NO;
  if (opt->transp == MDC_YES) {
-   printf("\n\tTransparent color [%u]? ",opt->transp_color);
+   MdcPrntScrn("\n\tTransparent color [%u]? ",opt->transp_color);
    if (!MdcPutDefault(mdcbufr)) opt->transp_color = (Uint8)atoi(mdcbufr);
  }
- printf("\n\tBackground  color [%u]? ",opt->bground_color);
+ MdcPrntScrn("\n\tBackground  color [%u]? ",opt->bground_color);
  if (!MdcPutDefault(mdcbufr))   opt->bground_color= (Uint8)atoi(mdcbufr);
- printf("\n");
+ MdcPrntScrn("\n");
  MdcPrintLine('-',MDC_FULL_LENGTH);
 }
 
@@ -795,11 +824,11 @@ void MdcGetGifOpt(FILEINFO *fi, MDC_GIFOPT *opt)
 int MdcWriteGifHeader(FILEINFO *fi, MDC_GIFOPT *opt)
 {
   MDC_GIFHEADER gh;
-  int bits = 8;  /* we only write 8-bit gifs */
+  unsigned int bits = 8;  /* we only write 8-bit gifs */
 
   /* fill the header struct */
   memset(&gh,0,MDC_GIF_GH_SIZE);
-  memcpy(gh.sig,MDC_GIFSIG,6);
+  memcpy(gh.sig,MDC_GIF89_SIG,6);
   gh.screenwidth=(Int16)fi->mwidth;
   gh.screenheight=(Int16)fi->mheight;
   gh.background=opt->bground_color;
@@ -816,7 +845,7 @@ int MdcWriteGifHeader(FILEINFO *fi, MDC_GIFOPT *opt)
   fwrite((char *)&gh,1,MDC_GIF_GH_SIZE,fi->ofp);
 
   /* write the colour map */
-  if (fi->palette != NULL) fwrite(fi->palette,1,3*(1<<bits),fi->ofp);
+  if (fi->palette != NULL) fwrite(fi->palette,1U,3U*(1U<<bits),fi->ofp);
 
   return(ferror(fi->ofp));
 }
@@ -1001,7 +1030,7 @@ int MdcWriteCommentBlock(FILEINFO *fi, const char *comment)
                 }
                 else {
                         fputc(n,fi->ofp);
-                        fwrite(comment,1,n,fi->ofp);
+                        fwrite(comment,1,(unsigned)n,fi->ofp);
                         fputc(0,fi->ofp);
                         n=0;
                 }
@@ -1076,7 +1105,7 @@ void MdcInitTable(Int16 min_code_size)
 void MdcFlush(FILE *fp, Int16 n)
 {
         fputc(n,fp);
-        fwrite(code_buffer,1,n,fp);
+        fwrite(code_buffer,1,(unsigned)n,fp);
 }
 
 /* write a code to the code buffer */
