@@ -12,11 +12,11 @@
  *                MdcWriteNIFTI()           - Write NIFTI file             *
  *                                                                         *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-/* $Id: m-nifti.c,v 1.20 2007/09/11 13:24:36 enlf Exp $
+/* $Id: m-nifti.c,v 1.25 2008/03/06 09:37:59 enlf Exp $
  */
 
 /*
-   Copyright (C) 1997-2007 by Erik Nolf
+   Copyright (C) 1997-2008 by Erik Nolf
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -163,7 +163,7 @@ const char *MdcReadNIFTI(FILEINFO *fi)
   for (i=0; i<8; i++) { 
      fi->dim[i] = nim->dim[i];
      fi->pixdim[i] = nim->pixdim[i];
-     if ((i > 0) && (fi->pixdim[i])) fi->pixdim[0] = i-1;
+     if ((i > 0) && (fi->pixdim[i] > 0.)) fi->pixdim[0] = i; /* one-based */
   }
 
   /* catch special case for single image */
@@ -298,7 +298,7 @@ const char *MdcWriteNIFTI(FILEINFO *fi)
   struct nifti_1_header nhdr;
   nifti_image *nim;
   znzFile fp=NULL;
-  char *bname;
+  char *bname, *pext;
   int i, n, ret, nvox, FREE;
   IMG_DATA *id;
   Int8 saved_norm_over_frames=MDC_NORM_OVER_FRAMES;
@@ -388,6 +388,12 @@ const char *MdcWriteNIFTI(FILEINFO *fi)
   sprintf(nim->descrip,"%.35s",fi->study_descr);
 
   bname = nifti_makebasename(fi->opath);
+  if (bname == NULL) return("NIFTI Base filename allocation failed");
+
+  /* get rid of any extension left, like .img.hdr -> .img remains */
+  /* which would result in a misappropriate two file nifti output */
+  pext = strchr(bname,'.'); if (pext != NULL) pext[0]='\0';    
+
   if (MDC_FILE_OVERWRITE == MDC_YES) {
     ret = nifti_set_filenames(nim,bname,0,1);
   }else{
@@ -488,14 +494,9 @@ const char *MdcWriteNIFTI(FILEINFO *fi)
                          type = BIT16_S; FREE=MDC_YES;
          }
        }else if (fi->diff_scale) {
-         switch (id->type) {
-           case FLT32  : buf = id->buf;
-                         type = FLT32; FREE=MDC_NO;
-                         break;
-           default     : buf = MdcGetImgFLT32(fi,i);
-                         type = FLT32; FREE=MDC_YES;
-                         break;
-         }
+           /* rescale to get single global factor */
+           buf = MdcGetImgFLT32(fi,i);
+           type = FLT32; FREE=MDC_YES;
        }else{
          /* all (or most) types supported */
          buf = id->buf; FREE=MDC_NO;
