@@ -21,11 +21,11 @@
  *                MdcContrastRemap()         - Apply contrast remapping    *
  *                                                                         *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-/* $Id: m-transf.c,v 1.29 2007/05/21 20:16:15 enlf Exp $
+/* $Id: m-transf.c,v 1.33 2008/03/06 08:17:38 enlf Exp $
  */
 
 /*
-   Copyright (C) 1997-2007 by Erik Nolf
+   Copyright (C) 1997-2008 by Erik Nolf
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -305,8 +305,8 @@ char *MdcCropImages(FILEINFO *fi, MDC_CROP_INFO *ecrop)
 
   /* set global parameters */
   new->number = cur->number;
-  new->mwidth = crop->width;   new->pixdim[1] = crop->width;
-  new->mheight= crop->height;  new->pixdim[2] = crop->height;
+  new->mwidth = crop->width;   new->dim[1] = crop->width;
+  new->mheight= crop->height;  new->dim[2] = crop->height;
 
   if (!MdcGetStructID(new,new->number)) {
     MdcCleanUpFI(new); MdcFree(new);
