@@ -12,11 +12,11 @@
  *                MdcEditFI()               - Edit FILEINFO structure      *
  *                                                                         *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-/* $Id: m-vifi.c,v 1.33 2007/05/21 20:16:15 enlf Exp $
+/* $Id: m-vifi.c,v 1.35 2008/01/23 21:28:37 enlf Exp $
  */
 
 /*
-   Copyright (C) 1997-2007 by Erik Nolf
+   Copyright (C) 1997-2008 by Erik Nolf
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -356,7 +356,7 @@ char *MdcEditFI(FILEINFO *fi)
       id->slice_width  = slice_width;
       id->slice_spacing= slice_spacing;
 
-      MdcFillImgPos(fi,i,i%fi->dim[3],(float)0.0);
+      MdcFillImgPos(fi,i,fi->dim[3]==0 ? 0 : i%fi->dim[3],(float)0.0);
       MdcFillImgOrient(fi,i);
    }
 
