@@ -10,11 +10,11 @@
  * Functions    : main()      - Main XMedCon routine                       *
  *                                                                         *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-/* $Id: xmedcon.c,v 1.26 2007/05/21 20:16:18 enlf Exp $
+/* $Id: xmedcon.c,v 1.28 2008/01/23 21:28:37 enlf Exp $
  */
 
 /*
-   Copyright (C) 1997-2007 by Erik Nolf
+   Copyright (C) 1997-2008 by Erik Nolf
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -94,6 +94,11 @@ int main(int argc, char *argv[])
     mdcrawinput.abs_offset = NULL;
     MdcInitRawPrevInput();
 
+#ifndef GTKONE
+    /* preserve POSIX locale: decimal = point */
+    gtk_disable_setlocale();
+#endif
+
     /* initialize the GTK/GDK_RGB engines */
     gtk_init(&argc, &argv);
     gdk_rgb_init();
