@@ -12,11 +12,11 @@
  * Functions    : XMdcMenusGetMain()      - Get main menu                  *
  *                                                                         *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-/* $Id: xmnuftry.c,v 1.34 2007/11/14 23:09:43 enlf Exp $
+/* $Id: xmnuftry.c,v 1.37 2008/01/23 21:28:37 enlf Exp $
  */
 
 /*
-   Copyright (C) 1997-2007 by Erik Nolf
+   Copyright (C) 1997-2008 by Erik Nolf
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -55,7 +55,7 @@ static GtkItemFactoryEntry menu_items[] = {
   {(char *)"/File/Raw Predef Save", (char *)"<control>I"
                                        , XMdcRawPredefSelSave, 0, NULL },
   {(char *)"/File/SepB", NULL, (void(*)())NULL, 0, (char *)"<Separator>" },
-  {(char *)"/File/Info Show", (char *)"<control>I", XMdcShowFileInfo, 0, NULL},
+  {(char *)"/File/Info Show", (char *)"<control>G", XMdcShowFileInfo, 0, NULL},
   {(char *)"/File/Info Edit", (char *)"<control>F", XMdcEditFileInfo, 0, NULL},
   {(char *)"/File/SepC", NULL, (void(*)())NULL, 0, (char *)"<Separator>" },
   {(char *)"/File/Save", (char *)"<control>S", XMdcFileSelSave
@@ -128,7 +128,7 @@ static GtkItemFactoryEntry menu_items[] = {
   {(char *)"/Images/Matrix/Square Pwr2",NULL,XMdcTransformImages
                                            ,MDC_TRANSF_SQR2,NULL},
   {(char *)"/_Options", NULL,  (void(*)())NULL, 0, (char *)"<Branch>"},
-  {(char *)"/Options/MedCon", "<control>M", XMdcOptionsMedconSel, 0, NULL},
+  {(char *)"/Options/MedCon", (char *)"<control>M",XMdcOptionsMedconSel,0,NULL},
   {(char *)"/Options/SepE", NULL, (void(*)())NULL, 0, (char *)"<Separator>" },
   {(char *)"/Options/Render", (char *)"<control>R",XMdcOptionsRenderSel,0,NULL},
   {(char *)"/Options/Labels", (char *)"<control>L", XMdcOptionsLabelSel,0,NULL},
@@ -152,8 +152,6 @@ void XMdcMenusGetMain(GtkWidget *window, GtkWidget **menubar)
 {
     guint nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);
     GtkItemFactory *factory;
-
-#ifdef GTKONE
     GtkAccelGroup *accel_group;
 
     accel_group = gtk_accel_group_new();
@@ -162,13 +160,10 @@ void XMdcMenusGetMain(GtkWidget *window, GtkWidget **menubar)
 
     gtk_item_factory_create_items(factory, nmenu_items, menu_items, NULL);
 
+#ifdef GTKONE
     gtk_accel_group_attach(accel_group, GTK_OBJECT(window));
 #else
-
-    factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", NULL);
-
-    gtk_item_factory_create_items(factory, nmenu_items, menu_items, NULL);
-
+    gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);
 #endif
 
     if (menubar)
